package org.capcaval.lafabrique.classgen;

import java.lang.reflect.Field;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import org.capcaval.lafabrique.classgen._test.TestClass;
import org.capcaval.lafabrique.classgen.converters.FloatToStringLiteralConverter;
import org.capcaval.lafabrique.classgen.converters.IntegerToStringLiteralConverter;
import org.capcaval.lafabrique.classgen.converters.LongToStringLiteralConverter;
import org.capcaval.lafabrique.classgen.converters.StringToStringLiteralConverter;
import org.capcaval.lafabrique.converter.Converter;
import org.capcaval.lafabrique.converter.ConverterManager;
import org.capcaval.lafabrique.file.FileTools;
import org.capcaval.lafabrique.lang.ArrayTools;
import org.capcaval.lafabrique.lang.ClassTools;
import org.capcaval.lafabrique.lang.StringMultiLine;
import org.capcaval.lafabrique.lang.StringTools;


public class ClassGenTools {

	public static Attribute<?>[] getAttributes(Object object) {
		List<Attribute<?>> list = new ArrayList<>();
		for(Field field : object.getClass().getDeclaredFields()){
			Object value = null;
			try {
				field.setAccessible(true);
				value = field.get(object);
			} catch (Exception e) {
				e.printStackTrace();
			} 
			// allocate a new attribute
			Attribute<?> attribute = new Attribute<>(field.getName(), field.getType(), value);
			// keep the value
			list.add(attribute);
		}
		
		return list.toArray(new Attribute[0]);
	}

	public static void generateClass(Path path, String fullClassName, Object instance, Converter<?, String>... converterArray) {
		
		
		
		// first get all the attribute values
		Attribute<?>[] attributeArray = getAttributes(instance);
		
		// get the class name
		String className = ClassTools.getClassNameFromFullName(fullClassName);

		// get the package name
		String packageName = ClassTools.getPackageNameFromFullName(fullClassName);
		
		// parent name
		Class<?> parentType = instance.getClass();
		
		generateClass(path, className, packageName, parentType, attributeArray, converterArray);
	}

	private static void generateClass(Path path, String className, String packageName, Class<?> parentType, Attribute<?>[] attributeArray, Converter<?, String>... converterArray) {
		StringMultiLine str = new StringMultiLine();
		ConverterManager cm = new ConverterManager(
				new FloatToStringLiteralConverter(),
				new IntegerToStringLiteralConverter(),
				new LongToStringLiteralConverter(),
				new StringToStringLiteralConverter());
		
		
		// add extra converters if any
		for(Converter<?, String> converter : converterArray){
			cm.addConverter(converter);
		}
		
		// add the package if any
		if((packageName != null)&&(packageName.length()>0)){
			str.addLine("package " + packageName + ";");}
		
		str.addLine("");
		
		// add all the import
//		for(Attribute<?> att : attributeArray){
//			// add only non primitive types
//			if(att.getType().isPrimitive() == false){
//				str.addLine("import " + att.getType().getName() + ";");}
//		}
		// Add the parent
		str.addLine("import " + parentType.getName() + ";");
		
		str.addLine("");
		
		// create the new class
		str.addLine("public class " + className +" extends " + parentType.getSimpleName() + "{");
		
		// add the constructor
		str.addLine("\tpublic " + className + "(){");
		for(Attribute<?> att : attributeArray){
			if(att.value != null){
				Converter<Object,String> converter = (Converter<Object,String>)cm.getConverter(att.getType(), String.class);
				
				if(converter == null){
					throw new RuntimeException("Does not have a string converter for type : " + att.getType() + " . Please add one inside your ClassGenTools");
				}
				
				String valStr = converter.convert(att.value);
				str.addLine("\t\tthis."+att.name + " = " + valStr + ";");
			}
		}
		str.addLine("\t}");
		
		str.addLine("}");
		
		// build path name
		//String filePath = path.toString() + "/" +
		String fileName = ClassTools.getFileNameFromClassName(packageName + "." +className);
		String fullName = path.toString()+"/"+fileName;
		
		FileTools.writeFile(fullName, str.toString());
	}
}
